@@ -1,8 +1,6 @@
 'use strict';
 
 const moment = require( "moment" );
-const fs = require( "fs" );
-const commandExistsSync = require( "command-exists" ).sync;
 
 // Settings, Globals and Constants
 let settings = require( "./cmd4Settings" );
@@ -239,6 +237,10 @@ class Cmd4Accessory
                                     );
       }
 
+      // Add the global constants to the listOfConstants
+      if ( this.parentInfo && this.parentInfo.globalConstants != null )
+         this.processConstants( this.parentInfo.globalConstants );
+
       // Direct if polling should be set or false.
       // You cannot copy polling from the parent because you would be copying the array
       // of polled characteristics that the child does not have, or turning on polling
@@ -1237,68 +1239,7 @@ class Cmd4Accessory
       if ( typeof state_cmd != "string" )
          throw new Error( `No state_cmd for: "${ this.displayName }".` );
 
-
-      // Split the state_cmd into words.
-      let cmdArray = state_cmd.match(/\S+/gi);
-
-      // Assume no words
-      let arrayLength = 0;
-
-      // Get the number of words
-      if ( cmdArray )
-         arrayLength = cmdArray.length;
-
-      // Check that the state_cmd is valid.
-      // The first word must be in the persons path
-      // The second word, if it exists must be a file
-      // and have the correct path.
-      switch ( arrayLength )
-      {
-         case 0:
-            this.log.error( `No state_cmd given` );
-            return false;
-         default:
-         {
-            let checkFile = cmdArray[ arrayLength -1 ];
-
-            try {
-               fs.accessSync( checkFile, fs.F_OK );
-               // File exists - OK
-
-            } catch ( e ) {
-               // It isn't accessible
-               throw new Error( `The file: "${ checkFile }" does not exist, It is highly likely the state_cmd will fail. Hint: Do not use wildcards that would normally be expanded by a shell.` );
-            }
-         }
-         // Purposely fall through to check the command as well
-         // break omitted
-         case 1:
-         {
-            let checkCmd = cmdArray[ 0 ];
-
-            // if the lone command is a path to a command
-            // Then check if it exists, oTherwise check if it is
-            // in Their path.
-            if ( checkCmd.charAt( 0 ) == "/" || (
-                  checkCmd.length > 1 &&
-                  checkCmd.charAt( 0 ) == "." &&
-                  checkCmd.charAt( 1 ) == "/" )
-               )
-            {
-               try {
-                  fs.accessSync( checkCmd, fs.F_OK );
-                  // File exists - OK
-               } catch ( e ) {
-                  throw new Error( `The file: "${ checkCmd }" does not exist, It is highly likely the state_cmd will fail. Hint: Do not use wildcards that would normally be expanded by a shell.` );
-               }
-            } else
-            {
-               if ( ! commandExistsSync( checkCmd ) )
-                  throw new Error( `The file: "${ checkCmd }" does not exist, It is highly likely the state_cmd will fail. Hint: Do not use wildcards that would normally be expanded by a shell.` );
-            }
-            break;
-         }
-      }
+      // This was messy, did not like spaces. let the user be the judge of it.
       return true;
    }
 
@@ -1321,11 +1262,8 @@ class Cmd4Accessory
       let accTypeEnumIndex = rcDirective.accTypeEnumIndex;
 
       // Do not update the stored values as it is being restored from cache
-      this.log.debug( `Checking to see if value should be set ${ parseConfigShouldUseCharacteristicValues }` );
       if ( parseConfigShouldUseCharacteristicValues == false )
          return;
-      //this.log.debug( `Going to set its value` );
-
 
       if ( Object.keys( CMD4_ACC_TYPE_ENUM.properties[ accTypeEnumIndex ].validValues ).length > 0 )
       {
@@ -1349,108 +1287,97 @@ class Cmd4Accessory
       this.cmd4Storage.setStoredValueForIndex( accTypeEnumIndex, properValue );
    }
 
-   processRequires( requires )
+   processRequires( requiresArray )
    {
-      if ( Array.isArray ( requires ) )
+      if ( ! Array.isArray ( requiresArray ) )
+         throw new Error( `requires must be an array of { "require": "some requires string" }` );
+
+      // Iterate over the groups of key/value constants in the array.
+      for ( let argIndex = 0; argIndex < requiresArray.length; argIndex++ )
       {
-          for ( let required in requires )
-             this.processRequires( requires[ required ] );
+         let argEntry = requiresArray[ argIndex ];
 
-         return;
+         if ( argEntry.require == undefined )
+            throw new Error( `Requires definition at index: "${ argIndex }" has no "require":"SomeRequire"` );
+
+         let required = argEntry.require;
+
+         if ( typeof required != "string" )
+            throw new Error( `Requires definition: "${ required }"  must be a string.` );
+         required = this.replaceConstantsInString( required );
+
+         if ( settings.cmd4Dbg ) this.log.debug( `Requiring ${ required }` );
+
+         require( required );
       }
-      if ( isJSON( requires ) )
-      {
-          // I assume only 1, but you know about assuming ...
-         for ( let key in requires )
-         {
-            let required = requires[ key ] ;
-
-            if ( typeof required != "string" )
-               throw new Error( `Requires definition: "${ required }"  must be a string.` );
-
-
-            if ( settings.cmd4Dbg ) this.log.debug( `Requiring ${ required }` );
-
-            require( required );
-         }
-         return;
-      }
-      throw new Error( `Requires must be an array of/or list of key/value pairs: "${ requires }".` );
    }
 
-   processConstants( constantArg )
+   processConstants( constantsArgArray )
    {
-      if ( Array.isArray ( constantArg ) )
-      {
-         for ( let constant in constantArg )
-            this.processConstants( constantArg[ constant ] );
+      if ( ! Array.isArray ( constantsArgArray ) )
+         throw new Error( `Constants must be an array of { "key": "\${SomeKey}", "value": "some replacement string" }` );
 
-         return;
+
+      // Iterate over the groups of key/value constants in the array.
+      for ( let argIndex = 0; argIndex < constantsArgArray.length; argIndex++ )
+      {
+         let argEntry = constantsArgArray[ argIndex ];
+
+         if ( argEntry.key == undefined )
+            throw new Error( `Constant definition at index: "${ argIndex }" has no "key":"\${SomeKey}"` );
+
+         if ( argEntry.value == undefined )
+            throw new Error( `Constant definition at index: "${ argIndex }" has no "value":"Some replacement string` );
+
+         let keyToAdd = argEntry.key;
+         let valueToAdd = argEntry.value;
+         if ( ! keyToAdd.startsWith( "${" ) )
+            throw new Error( `Constant definition for: "${ keyToAdd }" must start with "\${" for clarity.` );
+
+         if ( ! keyToAdd.endsWith( "}" ) )
+            throw new Error( `Constant definition for: "${ keyToAdd }" must end with "}" for clarity.` );
+
+         // remove any leading and trailing single quotes
+         // so that using it for replacement will be easier.
+         valueToAdd.replace(/^'/, "")
+         valueToAdd.replace(/'$/, "")
+
+         this.listOfConstants[ keyToAdd ] = valueToAdd;
       }
-      if ( isJSON( constantArg ) )
-      {
-         // I assume only 1, but you know about assuming ...
-         for ( let key in constantArg )
-         {
-            let keyToAdd = key ;
-            let valueToAdd = constantArg[ key ] ;
-            if ( ! keyToAdd.startsWith( "${" ) )
-               throw new Error( `Constant definition for: "${ keyToAdd }" must start with "\${" for clarity.` );
-
-
-            if ( ! keyToAdd.endsWith( "}" ) )
-               throw new Error( `Constant definition for: "${ keyToAdd }" must end with "}" for clarity.` );
-
-
-            // remove any leading and trailing single quotes
-            // so that using it for replacement will be easier.
-            valueToAdd.replace(/^'/, "")
-            valueToAdd.replace(/'$/, "")
-
-            this.listOfConstants[ keyToAdd ] = valueToAdd;
-        }
-        return;
-     }
-
-     throw new Error( `Constants must be an array of/or list of key/value pairs: "${ constantArg }".` );
    }
 
-   processVariables( variables )
+   processVariables( variablesArgArray )
    {
-      if ( Array.isArray ( variables ) )
+      if ( ! Array.isArray ( variablesArgArray ) )
+         throw new Error( `Variables must be an array of { "key": "\${SomeKey}", "value": "some replacement string" }` );
+
+
+      // Iterate over the groups of key/value constants in the array.
+      for ( let argIndex = 0; argIndex < variablesArgArray.length; argIndex++ )
       {
-         for ( let variable in variables )
-            this.processConstants( variables[ variable ] );
+         let argEntry = variablesArgArray[ argIndex ];
 
-         return;
+         if ( argEntry.key == undefined )
+            throw new Error( `Variable definition at index: "${ argIndex }" has no "key":"\${SomeKey}"` );
+
+         if ( argEntry.value == undefined )
+            throw new Error( `Variable definition at index: "${ argIndex }" has no "value":"Some replacement string` );
+
+         let keyToAdd = argEntry.key;
+         let valueToAdd = argEntry.value;
+         if ( ! keyToAdd.startsWith( "${" ) )
+            throw new Error( `Variable definition for: "${ keyToAdd }" must start with "\${" for clarity.` );
+
+         if ( ! keyToAdd.endsWith( "}" ) )
+            throw new Error( `Variable definition for: "${ keyToAdd }" must end with "}" for clarity.` );
+
+         // remove any leading and trailing single quotes
+         // so that using it for replacement will be easier.
+         valueToAdd.replace(/^'/, "")
+         valueToAdd.replace(/'$/, "")
+
+         this.listOfVariables[ keyToAdd ] = valueToAdd;
       }
-      if ( isJSON( variables ) )
-      {
-         // I assume only 1, but you know about assuming ...
-         for ( let key in variables )
-         {
-            let keyToAdd = key ;
-            let valueToAdd = variables[ key ] ;
-            if ( ! keyToAdd.startsWith( "${" ) )
-               throw new Error( `Variable definition for: "${ keyToAdd }" must start with "\${" for clarity.` );
-
-            if ( ! keyToAdd.endsWith( "}" ) )
-               throw new Error( `Variable definition for: "${ keyToAdd }" must end with "}" for clarity.` );
-
-            // remove any leading and trailing single quotes
-            // so that using it for replacement will be easier.
-            valueToAdd.replace(/^'/, "")
-            valueToAdd.replace(/'$/, "")
-
-            // The resultant variable may have Constants to be replaced.
-            let value = this.replaceConstantsInString( valueToAdd );
-
-            this.listOfVariables[ keyToAdd ] = value;
-         }
-         return;
-      }
-
-      throw new Error( `Variables must be an array of/or list of key/value pairs: "${ variables }".` );
    }
 
    accessoryTypeConfigToCmd4Accessories( config, parentInfo )
@@ -1537,7 +1464,7 @@ class Cmd4Accessory
                     throw new Error( `Unknown device type: "${ value }" given in: "${ this.displayName }".` );
 
                   // warn now
-                  this.log.warn( `The config.json Cmd4 device type: ${ value } is lowerCase.  It should be: ${ rcValue.deviceName }. In the near future this will be an error for homebridge-ui integration.\nTo remove this Warning, Please fix your config.json.` );
+                  this.log.warn( `The config.json Cmd4 device type: ${ value } is lowerCase.  It should be: ${ rcValue.deviceName }. In the near future this will be an error for homebridge-UI integration.\nTo remove this Warning, Please fix your config.json.` );
 
                   this.type = rcValue.deviceName;
                }
@@ -1606,6 +1533,9 @@ class Cmd4Accessory
 
                break;
             case constants.QUEUETYPES:
+               this.log.warn( `Warning: ${ key } will soon been deprecated at the accessory level. Please move it to where "Platform: "Cmd4" is located in your config.json.` );
+               this.log.warn( `This message will disappear when you have done so.` );
+
                parseAddQueueTypes( this.log, value );
 
                break;
@@ -1631,6 +1561,7 @@ class Cmd4Accessory
             case constants.POLLING:
                // Do not parse it yet as characteristics must be set first.
                this.polling = value;
+
                break;
             case "Cmd4_Mode":
             case "cmd4_Mode":
@@ -1656,6 +1587,7 @@ class Cmd4Accessory
                break;
             case constants.STATE_CMD_SUFFIX:
                // This gets added after any Get/Set <value>
+               // We replace any constants already defined
                this.state_cmd_suffix = value;
 
                break;
@@ -1665,18 +1597,30 @@ class Cmd4Accessory
 
                break;
             case constants.STORAGE:
+               this.log.warn( `Warning: ${ key } will soon been deprecated at the Platform level. It was always meant to just be a fakegato config option only.` );
+               this.log.warn( `This message will disappear when you have done so.` );
+
                this.storage = value;
 
                break;
             case constants.STORAGEPATH:
+               this.log.warn( `Warning: ${ key } will soon been deprecated at the Platform level. It was always meant to just be a fakegato config option only.` );
+               this.log.warn( `This message will disappear when you have done so.` );
+
                this.storagePath = value
 
                break;
             case constants.FOLDER:
+               this.log.warn( `Warning: ${ key } will soon been deprecated at the Platform level. It was always meant to just be a fakegato config option only.` );
+               this.log.warn( `This message will disappear when you have done so.` );
+
                this.folder = value
 
                break;
             case constants.KEYPATH:
+               this.log.warn( `Warning: ${ key } will soon been deprecated at the Platform level. It was always meant to just be a fakegato config option only.` );
+               this.log.warn( `This message will disappear when you have done so.` );
+
                this.keyPath = value
 
                break;
@@ -1690,6 +1634,9 @@ class Cmd4Accessory
 
                break;
             case constants.CONSTANTS:
+               this.log.warn( `Warning: ${ key } will soon been deprecated at the Accessory level. Please move it to where "Platform: "Cmd4" is located in your config.json.` );
+               this.log.warn( `This message will disappear when you have done so.` );
+
                this.processConstants( value );
 
                break;
@@ -1712,6 +1659,9 @@ class Cmd4Accessory
 
                break;
             case constants.ALLOWTLV8:
+               this.log.warn( `Warning: ${ key } will soon been deprecated at the Accessory level. Please move it to where "Platform: "Cmd4" is located.` );
+               this.log.warn( `This message will disappear when you have done so.` );
+
                this.allowTLV8 = value;
                break;
             default:
@@ -1766,9 +1716,15 @@ class Cmd4Accessory
 
       // Handle seperation of strings of state_cmd for a suffix
       if ( this.state_cmd_suffix )
-         this.state_cmd_suffix = " " + this.state_cmd_suffix;
-      else
+      {
+         if ( typeof this.state_cmd_suffix != "string" )
+            throw new Error( `state_cmd_suffix must be a string: "${ this.state_cmd_suffix }".` );
+
+         this.state_cmd_suffix = " " + this.replaceConstantsInString( this.state_cmd_suffix );
+      } else
+      {
          this.state_cmd_suffix = "";
+      }
 
       if ( this.typeIndex == CMD4_DEVICE_TYPE_ENUM.Television )
       {
@@ -1971,6 +1927,9 @@ class Cmd4Accessory
                      if ( this.queue && this.queue.queueName != value )
                         throw new Error( chalk.red( `Already defined Priority Polling Queue "${ this.queue.queueName }" for ${ this.displayName } cannot be redefined.` ) );
 
+                     this.log.warn( `Warning: ${ key } will soon been deprecated at the polling level. Please define it for the accessory: ${ this.displayName } as "queue": "${ value }"` );
+                     this.log.warn( `This message will disappear when you have done so.` );
+
                      this.queue = addQueue( this.log, value, constants.QUEUETYPE_WORM );
 
                      break;
@@ -1996,6 +1955,11 @@ class Cmd4Accessory
                   }
                   case constants.QUEUETYPES:
                   {
+                     // WHY IS THIS HERE AT ALL ??????????
+
+                     this.log.warn( `Warning: ${ key } will soon been deprecated at the accessory level. Please move it to where "Platform: "Cmd4" is located in your config.json.` );
+                     this.log.warn( `This message will disappear when you have done so.` );
+
                      parseAddQueueTypes( this.log, value );
                      // This whole record is not a characteristic polling entry
                      // continue to next ( via return )
